import pickle
import torch

def load_example_data(directory = "./"):
    flh = open(directory + "example.pkl", "rb")
    inp = pickle.load(flh)
    inp = {k: (torch.tensor(v) - 0.001) for k, v in zip(inp.keys(), inp.values())}
    return inp


from mobster.utils_mobster import *
import torch
import pyro.distributions as dist

def beta_lk(beta_a, beta_b, weights, K, data):
    lk = torch.ones(K, len(data))
    if K == 1:
        return torch.log(weights) + dist.Beta(beta_a, beta_b).log_prob(data)
    for k in range(K):
        lk[k, :] = torch.log(weights[k]) + dist.Beta(beta_a[k], beta_b[k]).log_prob(data)
    return lk

def final_lk(pareto, beta, weights):
    if len(beta.shape) == 1:
        dim0, dim1 = 1,beta.shape[0]
    else:
        dim0, dim1 = beta.shape[0], beta.shape[1]
    lk = torch.ones(1 + dim0, dim1)
    lk[0, :] = torch.log(weights[0]) + pareto
    lk[1:(1 + dim1), :] = torch.log(weights[1]) + beta
    return lk

def compute_likelihood_from_params(data, params, tail, tsum = True):

    theoretical_num_clones = get_theo_clones(data)
    clones_count = get_clones_counts(theoretical_num_clones)
    if tsum:
        lk = 0
    else :
        lk = [None] * len(theoretical_num_clones)
    for i,k in enumerate(data):

        if tail == 1:
            tmp = compute_likelihood_from_params_tail(data[k], params, i, theoretical_num_clones, clones_count)
            if tsum:
                tmp = log_sum_exp(tmp)
                lk += torch.sum(tmp)
            else:
                lk[i] = tmp

        else:
            tmp = compute_likelihood_from_params_no_tail(data[k], params, i, theoretical_num_clones, clones_count)
            if tsum:
                tmp = log_sum_exp(tmp)
                lk += torch.sum(tmp)
            else:
                lk[i] = tmp

    if not tsum:
        ks = data.keys()
        lk = {k:v for k,v in zip(ks, lk)}
    return lk



def compute_likelihood_from_params_tail(data, params, i, theo_clones, counts_clone):
    j = counts_clone[i]
    if theo_clones[i] == 2:
        beta = beta_lk(params['a_2'][:,j] * params['b_2'][:,j],
                       (1 - params['a_2'][:,j]) * params['b_2'][:, j],
                       params['param_weights_{}'.format(theo_clones[i])][j, :],
                       len(params['a_2'][:, j]),
                       data)
        pareto = dist.Pareto(torch.min(data) - 1e-5, params['ap']).log_prob(data)
        lk = final_lk(pareto, beta, params['param_tail_weights'][i, :])
    else:
        beta = beta_lk(params['a_1'][:, j] * params['b_1'][:, j],
                       (1 - params['a_1'][:, j]) * params['b_1'][:, j],
                       params['param_weights_{}'.format(theo_clones[i])][j,:],
                       len(params['a_1'][:, j]),
                       data)
        pareto = dist.Pareto(torch.min(data) - 1e-5, params['ap']).log_prob(data)
        lk = final_lk(pareto, beta, params['param_tail_weights'][i, :])
    return lk

def compute_likelihood_from_params_no_tail(data, params, i, theo_clones, counts_clone):
    j = counts_clone[i]
    if theo_clones[i] == 2:
        lk = beta_lk(params['a_2'][:, j] * params['b_2'][:, j],
                       (1 - params['a_2'][:, j]) * params['b_2'][:, j],
                       params['param_weights_{}'.format(theo_clones[i])][j, :],
                       len(params['a_2'][:, j]),
                       data)
    else:
        lk = beta_lk(params['a_1'][:, j] * params['b_1'][:, j],
                       (1 - params['a_1'][:, j]) * params['b_1'][:, j],
                       params['param_weights_{}'.format(theo_clones[i])][j, :],
                       len(params['a_1'][:, j]),
                       data)
    return lk



from scipy.stats import beta, pareto

import matplotlib as mpl
mpl.use('Agg')

import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import numpy as np


def plot_results(data, inf_res, bins=50, output = "results.png",fig_height = 10, fig_width = 8):

    all_params = inf_res["model_parameters"]
    plt.rcParams["figure.figsize"] = (fig_height, fig_width)
    tail = inf_res['run_parameters']['tail']
    fig, axs = plt.subplots(len(all_params))
    for i, kr in enumerate(data):

        axs[i].hist(data[kr].detach().numpy(), bins=bins, density=True, alpha=0.48)
        params = all_params[kr]

        karyos = list(data.keys())

        major = [int(str(i).split(":")[0]) for i in karyos]
        minor = [int(str(i).split(":")[1]) for i in karyos]
        theoretical_num_clones = [1 if (mn == 0 or mn == mj) else 2 for mj, mn in zip(major, minor)]
        assignment_probs = params["mixture_probs"]

        for j in range(tail, len(assignment_probs)):

            if theoretical_num_clones[i] == 1:
                cl = "tab:green" if (j - tail) < 1 else "tab:red"

            else:
                cl = "tab:green" if (j - tail) < 2 else "tab:red"

            a = params["beta_concentration1"][j - tail]
            b = params["beta_concentration2"][j - tail]

            x = np.linspace(0.05, 1, 200)

            p = beta.pdf(x, a, b) * assignment_probs[j]


            axs[i].plot(x, p, linewidth=3, color=cl)

        if tail == 1:
            alpha = params["tail_shape"]
            x = np.linspace(0.05, 1, 200)
            p = pareto.pdf(x, alpha, scale=params["tail_scale"]) * assignment_probs[0]
            axs[i].plot(x, p, linewidth=3, color="tab:pink")


    pink_patch = mpatches.Patch(color='tab:pink', label='Tail')
    red_patch = mpatches.Patch(color='tab:red', label='Subclonal')
    green_patch = mpatches.Patch(color='tab:green', label='Clonal')

    plt.legend(handles=[pink_patch, red_patch, green_patch])

    plt.savefig(output, dpi=300)


from pandas.core.common import flatten
from tqdm import trange
import torch
import numpy as np

def flatten_list(l):
    return list(flatten(l))


def log_sum_exp(args):
    c = torch.amax(args, dim=0)
    return c + torch.log(torch.sum(torch.exp(args - c), axis=0))


def get_theo_clones(data):
    karyos = list(data.keys())
    major = [int(str(i).split(":")[0]) for i in karyos]
    minor = [int(str(i).split(":")[1]) for i in karyos]
    theoretical_num_clones = [1 if (mn == 0 or mn == mj) else 2 for mj, mn in zip(major, minor)]
    return theoretical_num_clones

def get_clones_counts(theoretical_num_clones):
    uniques = list(set(theoretical_num_clones))
    uniques = {k : 0 for k in uniques}
    res = [None] * len(theoretical_num_clones)
    for i in range(len(theoretical_num_clones)):
        res[i] = uniques[theoretical_num_clones[i]]
        uniques[theoretical_num_clones[i]] += 1
    return(res)


def compute_entropy(params, tail):

    res = 0
    posts = params["cluster_probs"]
    for k in posts:
        posts_k = posts[k]
        if tail:
            posts_k = posts_k[1:]
        log_post_k = torch.log(posts_k + 0.000001)
        post_k_entr = posts_k * log_post_k
        post_k_entr = torch.sum(post_k_entr, axis = 0)
        post_k_entr = -1 * torch.sum(post_k_entr)
        res += post_k_entr
    return res



def format_parameters_for_export(data, params, tail):
    res = {k : 0 for k in data.keys()}
    theoretical_num_clones = get_theo_clones(data)
    clones_count = get_clones_counts(theoretical_num_clones)
    for i, k in enumerate(res):
        res[k] = format_parameters_for_export_aux(data, params,k, i, theoretical_num_clones, clones_count, tail)
    return res


def format_parameters_for_export_aux(data, params,k, i, theo_clones, counts_clone, tail):

    j = counts_clone[i]
    if theo_clones[i] == 2:
        beta_concentration1 = params['a_2'][:,j] * params['b_2'][:,j]
        beta_concentration2 = (1 - params['a_2'][:,j]) * params['b_2'][:, j]
    else:
        beta_concentration1 = params['a_1'][:, j] * params['b_1'][:, j]
        beta_concentration2 = (1 - params['a_1'][:, j]) * params['b_1'][:, j]

    mixture_weights = params['param_weights_{}'.format(theo_clones[i])][j, :].detach().numpy()
    if tail == 1:
        tail_weights = params['param_tail_weights'][i, :].detach().numpy()
        mixture_weights = mixture_weights * tail_weights[1]
        mixture_weights = np.insert(mixture_weights, 0, tail_weights[0])




    res = {"cluster_probs" : params["cluster_probs"][k].detach().numpy(),
           "cluster_assignments" : np.argmax(params["cluster_probs"][k].detach().numpy(), axis = 0),
           "mixture_probs" : mixture_weights,
          "beta_concentration1" : beta_concentration1.detach().numpy(),
          "beta_concentration2" : beta_concentration2.detach().numpy()
        }
    if tail == 1:
        res["tail_shape"] = params['ap'].detach().numpy()
        res["tail_scale"] = np.min(data[k].detach().numpy())

    return res


import pandas as pd
import numpy as np
import seaborn as sns
import tqdm

import pyro as pyro
import numpy as np
from pyro.infer.autoguide import AutoDelta
import pyro.poutine as poutine
import torch
import mobster
from mobster.utils_mobster import *
from mobster.stopping_criteria import *
import mobster.model_selection_mobster as ms
from mobster.calculate_posteriors import *

def fit_mobster(data, K, tail=1, purity=0.96, alpha_prior_sd=0.5, number_of_trials_clonal_mean=500.,number_of_trials_k=300.,
         prior_lims_clonal=[0.1, 100000.], prior_lims_k=[0.1, 100000.], stopping = ELBO_stopping_criteria, lr = 0.05, max_it = 5000, e = 0.01):

    model = mobster.model
    guide = mobster.guide

    svi = pyro.infer.SVI(model=model,
                     guide=guide,
                     optim=pyro.optim.Adam({"lr": lr}),
                     loss=pyro.infer.TraceGraph_ELBO())

    print('Running MOBSTER on {} karyotypes with {} subclones.'.format(len(data), K), flush=True)
    if tail == 1:
        print("Fitting a model with tail", flush = True)
    else:
        print("Fitting a model without tail", flush=True)
    params = {
        'K' : K,
        'tail' : tail,
        'purity' : purity,
        'alpha_prior_sd' : alpha_prior_sd,
        'number_of_trials_clonal_mean' : number_of_trials_clonal_mean,
        'number_of_trials_k' : number_of_trials_k,
        'prior_lims_clonal' : prior_lims_clonal,
        'prior_lims_k' : prior_lims_k
    }
    run(data, params, svi, stopping, max_it, e)

    params_dict = retrieve_params()

    print("Computing cluster assignements.", flush=True)
    params_dict = retrieve_posterior_probs(data, params_dict, tail)


    ### Caclculate information criteria
    print("Computing information criteria.", flush=True)
    likelihood = ms.likelihood(data, params_dict, tail)
    AIC = ms.AIC(data, params_dict, tail)
    BIC = ms.BIC(data, params_dict, tail)
    ICL = ms.ICL(data, params_dict, tail)

    params_dict = format_parameters_for_export(data, params_dict, tail)



    information_dict =  {"likelihood": likelihood.detach().numpy(),
                         "AIC": AIC.detach().numpy(),
                        "BIC" : BIC.detach().numpy(),
                        "ICL" : ICL.detach().numpy()}


    final_dict = {
        "information_criteria" : information_dict,
        "model_parameters" : params_dict,
        "run_parameters" : params
    }
    print("Done!", flush = True)

    return final_dict



def run(data, params, svi, stopping, max_it, e):

    N = ms.number_of_samples(data)
    data_dict = params.copy()
    data_dict["data"] = data
    pyro.clear_param_store()
    loss = new = svi.step(**data_dict)
    losses = []
    t = trange(max_it, desc='Bar desc', leave=True)
    for i in t:

        t.set_description('ELBO: {:.9f}  '.format(loss / N))
        t.refresh()

        loss = svi.step(**data_dict)
        losses.append(loss)

        old, new = new, loss

        if stopping(old, new, e):
            continue




def retrieve_params():
    param_names = pyro.get_param_store()
    res = {nms: pyro.param(nms) for nms in param_names}
    return res


import pyro as pyro
from pyro.infer import config_enumerate
import torch
from mobster.likelihood_calculation import *



@config_enumerate
def model(data, K=1, tail=1, purity=0.96, alpha_prior_sd=0.3, number_of_trials_clonal_mean=500., number_of_trials_k=300.,
         prior_lims_clonal=[0.1, 100000.], prior_lims_k=[0.1, 100000.]):

    """Hierarchical bayesian model for Subclonal Deconvolution from VAF

    This model deconvolves the signal from the Variant Allelic Frequency spectrum.


    Parameters
    ----------
    data : dictionary
    A dictionary with karyotypes as keys (written in this form major_allele:minor_allele) and
    as values float torch tensors with the VAF value
    K : int
    Number of subclonal clusters
    tail: int
    1 if inference is to perform with Pareto tail, 0 otherwise
    purity: float
    Previously estimated purity of the tumor
    alpha_prior_sd: float
    Prior standard deviation for the LogNormal distribution describing
    the shape of the Pareto Distribution
    number_of_trials_clonal_mean : int
    Number of trials for the clonal
    prior_lims_clonal : list
    Limits of the Uniform over the prior for the number of trials for the clonal clusters
    prior_lims_k : list
    Limits of the Uniform over the prior for the number of trials for the subclonal clusters

    """

    # Split major and minor allele
    karyos = list(data.keys())
    major = [int(str(i).split(":")[0]) for i in karyos]
    minor = [int(str(i).split(":")[1]) for i in karyos]

    # Initialize means as they are into conditional block
    betas_subclone_mean2 = 0
    betas_subclone_n_samples2 = 0
    betas_subclone_mean = 0
    betas_subclone_n_samples = 0

    # Here we calculate the theoretical number of clonal clusters, i.e. 1 if LOH of major=minor, 2 otherwise

    theoretical_num_clones = [1 if (mn == 0 or mn == mj) else 2 for mj, mn in zip(major, minor)]

    # Calculate the theoretical clonal means, whichc correspond just to major/tot (and minor/tot) * purity
    theoretical_clonal_means = [purity if (mn == 0 or mn == mj) else torch.tensor(
        [(mj / (mj + mn)) * purity, (mn / (mn + mj)) * purity]) for mj, mn in zip(major, minor)]
    # We do it separately for number of clonal cluster 1 and 2
    theoretical_clonal_means = [0.5 * purity if (mn == mj) else j for mj, mn, j in
                                zip(major, minor, theoretical_clonal_means)]

    # Count how many karyotypes we have in our dataset for each theoretical number of clones
    counts_clones = dict()
    for i in theoretical_num_clones:
        counts_clones[i] = counts_clones.get(i, 0) + 1

    # Prior over the mean of the alphas
    alpha_prior = pyro.sample('u', dist.Uniform(0.5, 5))

    # We enter the karyotype plate
    # We may think about tensorizing it
    for kr in pyro.plate("kr", len(karyos)):

        # sample the tail slope
        alpha = pyro.sample('alpha_{}'.format(kr), dist.LogNormal(torch.log(alpha_prior), alpha_prior_sd))

        # We construct two different computational graphs for cases with 2 and 1 clonal clusters
        if theoretical_num_clones[kr] == 2:

            # mixture weights for the beta parameters with theoretical number of clones 2
            weights_2 = pyro.sample('weights_{}'.format(kr), dist.Dirichlet(torch.ones(K + 2)))

            # Here we initialize both the clonal and the subclonal beta clusters
            with pyro.plate("clones_{}".format(kr), 2 + K):

                # Subclonal clusters are initialized halfway between zero and the smallest clonal cluster
                if K == 0:
                    k_means = torch.zeros(0)
                else:
                    k_means = (torch.min(theoretical_clonal_means[kr] - 0.05) / K) * torch.arange(1,K+1)
                # Number of sucessful trials for beta means prior
                bm_12 = torch.tensor(
                    flatten_list([theoretical_clonal_means[kr].tolist(), k_means.tolist()])) * number_of_trials_clonal_mean
                # Number of unsucessful trials for beta means prior
                bm_22 = number_of_trials_clonal_mean - bm_12
                # As we are writing a bayesian model, beta clonal means prior are actually around
                # the theoretical values
                betas_subclone_mean2 = pyro.sample('beta_clone_mean_{}'.format(kr), dist.Beta(bm_12, bm_22))

                # Here we prepare the prior for the beta number of samples
                bns_12 = torch.tensor((2 * [prior_lims_clonal[0]]) + (K * [prior_lims_k[0]]))
                bns_22 = torch.tensor((2 * [prior_lims_clonal[1]]) + (K * [prior_lims_k[1]]))
                betas_subclone_n_samples2 = pyro.sample('beta_clone_n_samples_{}'.format(kr),
                                                        dist.Uniform(bns_12, bns_22))
        # The same as before but with number of clonal cluster equal to 1
        else:

            weights_1 = pyro.sample('weights_{}'.format(kr), dist.Dirichlet(torch.ones(K + 1)))
            # Pytorch kinda sucks sometimes, so some code here is pretty hard to read
            # Maybe to rewrite using only numpy arrays and not lists
            with pyro.plate("clones_{}".format(kr), 1 + K):

                if K == 0:
                    k_means = torch.zeros(0)
                else:
                    k_means = (theoretical_clonal_means[kr] - 0.05) / K *  torch.arange(1,K+1)
                bm_11 = torch.tensor(
                    flatten_list([theoretical_clonal_means[kr], k_means.tolist()])) * number_of_trials_clonal_mean
                bm_21 = number_of_trials_clonal_mean - bm_11
                betas_subclone_mean = pyro.sample('beta_clone_mean_{}'.format(kr), dist.Beta(bm_11, bm_21))
                bns_11 = torch.tensor((1 * [prior_lims_clonal[0]]) + (K * [prior_lims_k[0]]))
                bns_21 = torch.tensor((1 * [prior_lims_clonal[1]]) + (K * [prior_lims_k[1]]))
                betas_subclone_n_samples = pyro.sample('beta_clone_n_samples_{}'.format(kr),
                                                       dist.Uniform(bns_11, bns_21))

        # Tail vs no tail probability, Dirichlet priors can sometimes create problems, but no better solution
        tail_probs = pyro.sample('weights_tail_{}'.format(kr), dist.Dirichlet(torch.ones(2)))
        with pyro.plate('data_{}'.format(kr), len(data[karyos[kr]])):

        # Here we split again the computational graph in case of tail or no tail
            if (tail == 1):
                # The likelihood is different among the karyotypes classes
                if theoretical_num_clones[kr] == 2:
                    beta = beta_lk(betas_subclone_mean2 * betas_subclone_n_samples2,
                                   (1 - betas_subclone_mean2) * betas_subclone_n_samples2,
                                   weights_2, K + theoretical_num_clones[kr],
                                   data[karyos[kr]])
                    pareto = dist.Pareto(torch.min(data[karyos[kr]]) - 1e-5, alpha).log_prob(data[karyos[kr]])
                    pyro.factor("lik_{}".format(kr), log_sum_exp(final_lk(pareto, beta, tail_probs)).sum())


                else:
                    beta = beta_lk(betas_subclone_mean * betas_subclone_n_samples,
                                   (1 - betas_subclone_mean) * betas_subclone_n_samples,
                                   weights_1, K + theoretical_num_clones[kr],
                                   data[karyos[kr]])
                    pareto = dist.Pareto(torch.min(data[karyos[kr]]) - 1e-5, alpha).log_prob(data[karyos[kr]])
                    pyro.factor("lik_{}".format(kr), log_sum_exp(final_lk(pareto, beta, tail_probs)).sum())

            else:

                if theoretical_num_clones[kr] == 2:
                    pyro.factor("lik_{}".format(kr), torch.sum(log_sum_exp(beta_lk(betas_subclone_mean2 * betas_subclone_n_samples2,
                                                                       (1 - betas_subclone_mean2) * betas_subclone_n_samples2,
                                                                       weights_2, K + theoretical_num_clones[kr],
                                                                       data[karyos[kr]]))))
                else:
                    pyro.factor("lik_{}".format(kr), torch.sum(log_sum_exp(beta_lk(betas_subclone_mean * betas_subclone_n_samples,
                                                                       (1 - betas_subclone_mean) * betas_subclone_n_samples,
                                                                       weights_1,
                                                                       K + theoretical_num_clones[kr],
                                                                       data[karyos[kr]]))))






from mobster.likelihood_calculation import *


def retrieve_posterior_probs(data, parameters, tail):
    lks = compute_likelihood_from_params(data, parameters, tail, tsum = False)
    res = {k : 0 for k in data.keys()}
    for k in res:
        lks_k = lks[k]
        norm_fact = log_sum_exp(lks_k)
        res[k] = torch.exp(lks_k - norm_fact)
    parameters["cluster_probs"] = res
    return parameters



import pyro as pyro
import numpy as np
from pyro.infer import MCMC, NUTS, Predictive, HMC, config_enumerate
from pyro.infer.autoguide import AutoDelta, init_to_sample
import pyro.distributions as dist
import pyro.poutine as poutine
import torch
from torch.distributions import constraints

@config_enumerate
def guide(data, K=1, tail=1, purity=0.96, clonal_beta_var=1., alpha_prior_sd=0.3, number_of_trials_clonal_mean=100.,
          number_of_trials_clonal=900., number_of_trials_k=300., prior_lims_clonal=[1., 10000.],
          prior_lims_k=[1., 10000.]):
    """
    Parameters
    ----------
    param1 :
    The first parameter.
    param2 :
    The second parameter.

    """
    karyos = list(data.keys())
    major = [int(str(i).split(":")[0]) for i in karyos]
    minor = [int(str(i).split(":")[1]) for i in karyos]

    lk = np.zeros(len(karyos))

    theoretical_num_clones = [1 if (mn == 0 or mn == mj) else 2 for mj, mn in zip(major, minor)]
    theoretical_clonal_means = [(purity - 0.0001) if (mn == 0 or mn == mj) else torch.tensor(
        [(mj / (mj + mn)) * purity, (mn / (mn + mj)) * purity]) for mj, mn in zip(major, minor)]
    theoretical_clonal_means = [0.5 * purity if (mn == mj) else j for mj, mn, j in
                                zip(major, minor, theoretical_clonal_means)]

    counts_clones = dict()
    for i in theoretical_num_clones:
        counts_clones[i] = counts_clones.get(i, 0) + 1

    index_2 = [i for i, j in enumerate(theoretical_num_clones) if j == 2]
    index_1 = [i for i, j in enumerate(theoretical_num_clones) if j == 1]

    ap = pyro.param("ap", torch.tensor(1.5), constraint=constraints.interval(0.5, 5))

    alpha_prior = pyro.sample('u', dist.Delta(ap))

    idx1 = 0
    idx2 = 0

    weights_tail = pyro.param("param_tail_weights", 1 / torch.ones([len(karyos), 2]), constraint=constraints.simplex)
    weights_param_2 = pyro.param("param_weights_2", (1 / (K + 2)) * torch.ones([len(index_2), K + 2]),
                                 constraint=constraints.simplex)
    weights_param_1 = pyro.param("param_weights_1", (1 / (K + 1)) * torch.ones([len(index_1), K + 1]),
                                 constraint=constraints.simplex)

    for kr in pyro.plate("kr", len(karyos)):

        pyro.sample('alpha_{}'.format(kr), dist.LogNormal(torch.log(alpha_prior), alpha_prior_sd))

        if theoretical_num_clones[kr] == 2:

            pyro.sample('weights_{}'.format(kr), dist.Delta(weights_param_2[idx2]).to_event(1))

            # Mean parameter when the number of clonal picks is 2
            a_2_theo = torch.cat([theoretical_clonal_means[i] for i in index_2]).reshape([2, counts_clones[2]])
            max_lim_2 = torch.tensor([torch.min(theoretical_clonal_means[i]) for i in index_2])
            a_2_k = dist.Uniform(0.1, max_lim_2 - (0.3 * max_lim_2)).sample([K])

            # Number of trials parameter when the number of clonal picks is 2
            b_2_theo = torch.ones([2, len(index_2)]) * number_of_trials_clonal_mean
            b_2_k = torch.ones([K, len(index_2)]) * number_of_trials_k

            a21 = pyro.param('a_2',
                             torch.cat((a_2_theo, a_2_k)).reshape([2 + K, len(index_2)]),
                             constraint=constraints.unit_interval)
            a22 = pyro.param('b_2',
                             torch.cat((b_2_theo, b_2_k)).reshape([2 + K, len(index_2)]),
                             constraint=constraints.positive)

            with pyro.plate("clones_{}".format(kr), 2 + K):
                pyro.sample('beta_clone_mean_{}'.format(kr), dist.Delta(a21[:, idx2]))
                pyro.sample('beta_clone_n_samples_{}'.format(kr), dist.Delta(a22[:, idx2]))
            idx2 += 1

        else:
            pyro.sample('weights_{}'.format(kr), dist.Delta(weights_param_1[idx1]).to_event(1))

            # Mean parameter when the number of clonal picks is 1
            a_1_theo = torch.tensor([theoretical_clonal_means[i] for i in index_1]).reshape([1, counts_clones[1]])
            max_lim_1 = torch.tensor([np.min(theoretical_clonal_means[i]) for i in index_1])
            a_1_k = dist.Uniform(0.1, max_lim_1 - (0.3 * max_lim_1)).sample([K])

            # Number of trials parameter when the number of clonal picks is 1
            b_1_theo = torch.ones([1, len(index_1)]) * number_of_trials_clonal_mean
            b_1_k = torch.ones([K, len(index_1)]) * number_of_trials_k

            a11 = pyro.param('a_1',
                             torch.cat((a_1_theo, a_1_k)).reshape([1 + K, len(index_1)]),
                             constraint=constraints.unit_interval)
            a12 = pyro.param('b_1',
                             torch.cat((b_1_theo, b_1_k)).reshape([1 + K, len(index_1)]),
                             constraint=constraints.positive)

            with pyro.plate("clones_{}".format(kr), 1 + K):
                pyro.sample('beta_clone_mean_{}'.format(kr), dist.Delta(a11[:, idx1]))
                pyro.sample('beta_clone_n_samples_{}'.format(kr), dist.Delta(a12[:, idx1]))
            idx1 += 1

            # K = K + tail
        pyro.sample('weights_tail_{}'.format(kr), dist.Delta(weights_tail[kr]).to_event(1))




from setuptools import setup

#def readme():
#    with open('README.md') as f:
#        return f.read()

setup(name='HMOBSTER',
      version='0.0.7',
      description='Copy Number genotyping from single cell RNA sequencing',
      url='https://github.com/Militeee/hmobster',
      author='Salvatore Milite',
      author_email='militesalvatore@gmail.com',
      license='GPL-3.0',
      packages=['mobster'],
      install_requires=[
            'matplotlib>=3.1',
            'pandas>=1.0',
            'pyro-ppl>=0.4',
            'numpy>=1.18',
            'scikit-learn',
            'seaborn'

      ],
      include_package_data=True,
      #long_description=readme(),
      classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering :: Bio-Informatics'
      ],
      keywords='DNA sequencing clonal_evolution VAF',
      test_suite='nose.collector',
      tests_require=['nose'],
      zip_safe=False)


from mobster.likelihood_calculation import *
import numpy as np


def ICL(data, params, tail):
    bic = BIC(data, params, tail)
    entropy = compute_entropy(params, tail)
    return bic + entropy

def BIC(data, params, tail):
    lk = likelihood(data, params, tail)
    n_params = calculate_number_of_params(params)
    n = number_of_samples(data)
    return np.log(n) * n_params - 2 * lk


def AIC(data, params, tail):
    lk = likelihood(data, params, tail)
    n_params = calculate_number_of_params(params)
    return 2 * n_params - 2 * lk


def likelihood(data, params, tail):
    lk = compute_likelihood_from_params(data, params, tail)
    return lk


def calculate_number_of_params(params):

    res = 0
    for i in params:
        if type(params[i]) is dict:
            next
        else:
            res += np.prod(params[i].shape)
    return res


def number_of_samples(data):

    res = 0
    for k in data:
        res += np.prod(data[k].shape)
    return res


#__all__ = ['guide', 'model', 'stopping_criteria', 'calculate_posteriors',
#           'interface', 'example_loading', 'plotting', 'model_selection']

from mobster.guide_mobster import guide
from mobster.model_mobster import model
from mobster.interface_mobster import fit_mobster
from mobster.example_loading import *
from mobster.plotting_mobster import *
from mobster.stopping_criteria import *
